# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from pykickstart.parser import KickstartHandlers

class P2VDetector(KickstartHandlers):
	''' The P2VDector class is a subclass of the KickstartHandlers class from the pykickstart package.
It's purpose is to examine a kickstart file for the "p2v" directive.  If it finds the p2v flag it sets
the ksdata.p2vFlag attribute to True.  The purpose of the p2v flag is to signal anaconda that a kickstart
file is intended to be used for a physical to virtual conversion rather than a traditional install.'''

	def __init__ (self, ksdata):
		KickstartHandlers.__init__(self, ksdata)

		# reset all handlers so that all commands are ignored
		# execpt "p2v"
		self.resetHandlers()
		self.handlers["p2v"] = self.doP2V

		# Need to add these as they are not in the standard set of kickstart
		# parameters but will be part of OVS kickstart files.  Don't need to do
		# anything, just not barf when they are there
		self.handlers["ovsagent"] = None
		self.handlers["ovsmgmntif"] = None

		# These are parameters accepted in normal p2v kickstart files, so we need
		# to ignore them here
		self.handlers["target"] = None
		self.handlers["diskimage"] = None
		self.handlers["vm_options"] = None

	def doP2V (self, args):
		'''Handler for the "p2v" directive.  Sets ksdata.p2vFlag to True.'''
		# There should be no args to the "p2v" directive so no need to process
		self.ksdata.p2vFlag = True
