# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

import os, re
from pykickstart.parser import KickstartHandlers, KickstartValueError
from pykickstart.parser import KSOptionParser
from p2vdata.P2VDiskImageData import P2VDiskImageData

class P2VHandlers(KickstartHandlers):
        def __init__ (self, ksdata):
                KickstartHandlers.__init__(self, ksdata)

                # reset all handlers so that all commands are ignored
                self.resetHandlers()

		# Handle the commands specific to p2v operations.
		# These are defined here
		self.handlers["target"] = self.doTarget
		self.handlers["diskimage"] = self.doDiskImage
		self.handlers["vm_options"] = self.doVMOptions

		# Reset this one to the default from the KickstartHandlers class
		self.handlers["network"] = self.doNetwork

                # Need to add these as they are not in the standard set of kickstart
                # parameters but will be part of OVS kickstart files.  Don't need to do
                # anything, just not barf when they are there
                self.handlers["p2v"] = None 
                self.handlers["ovsagent"] = None
                self.handlers["ovsmgmntif"] = None

        def doTarget (self, args):
		op = KSOptionParser(lineno=self.lineno)
		op.add_option("--ovmmanager", dest="managertransfer", action="store_true",
			default=False)
		op.add_option("--scp", dest="scptransfer", action="store_true",
			default=False)
		op.add_option("--ftp", dest="ftptransfer", action="store_true",
			default=False)
		op.add_option("--host", dest="target", action="store", type="string", required=0)
		op.add_option("--user", dest="user", action="store", type="string", required=0)
		op.add_option("--passwd", dest="passwd", action="store", type="string", required=0)
		op.add_option("--dir", dest="dir", action="store", type="string", required=0)

		(opts, extra) = op.parse_args(args=args)

		self._setToObj(op, opts, self.ksdata)

		# Only makes sense to supply one type of transfer
		# if transfer via OVM manager was specified, blank out FTP or SCP
		if self.ksdata.managertransfer == True:
			self.ksdata.scptransfer = False
			self.ksdata.ftptransfer = False
			self.ksdata.user = None
			self.ksdata.passwd = None
			self.ksdata.dir = None

	def doDiskImage (self, args):
		op = KSOptionParser(lineno=self.lineno)
		op.add_option("--device", dest="path", action="store", type="string", required=1)
		op.add_option("--type", dest="type", action="store", type="string", required=1)
		(opts, extra) = op.parse_args(args=args)

		didata = P2VDiskImageData()
		self._setToObj(op, opts, didata)
		didata.device = os.path.basename(didata.path)

		ideregex = re.compile("xvd[a-z]*|hd[a-d]")
		if ideregex.search(didata.path):
			if not didata.type == "IDE":
				raise KickstartValueError, "Error at line %s: Device %s should have type \'IDE\'" %(self.lineno, didata.path)
		else:
			if not didata.type == "SCSI":
				raise KickstartValueError, "Error at line %s: Device %s should have type \'SCSI\'" %(self.lineno, didata.path)
		self.ksdata.diskimages.append(didata)

	def doVMOptions(self, args):
		op = KSOptionParser(lineno=self.lineno)
		op.add_option("--name", dest="vm_name", action="store", type="string", required=0)
		op.add_option("--mem", dest="vm_mem", action="store", type="string", required=0)
		op.add_option("--vcpus", dest="vcpus", action="store", type="string", required=0)
		op.add_option("--consolepasswd", dest="consolepasswd", action="store", type="string", required=0)
		(opts, extra) = op.parse_args(args=args)
		
		self._setToObj(op, opts, self.ksdata)
