# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

import pexpect, os, sys, socket, time, mimetypes, re
from SocketServer import BaseServer
from BaseHTTPServer import HTTPServer
from SimpleHTTPServer import SimpleHTTPRequestHandler
from OpenSSL import SSL
from threading import Thread, RLock

class SecureHTTPServer(HTTPServer, Thread):
	def __init__(self, server_address, HandlerClass, keyfile, certificate):
		BaseServer.__init__(self, server_address, HandlerClass)
		Thread.__init__(self)
		ctx = SSL.Context(SSL.SSLv23_METHOD)
		ctx.use_privatekey_file (keyfile)
		ctx.use_certificate_file(certificate)
		self.socket = SSL.Connection(ctx, socket.socket(self.address_family, self.socket_type))
		self.server_bind()
		self.server_activate()

	def run(self):
		print "Serving forever"
		self.serve_forever()
		print "Should not get here"

class SecureHTTPRequestHandler(SimpleHTTPRequestHandler):
	def setup(self):
		self.connection = self.request
		self.rfile = socket._fileobject(self.request, "rb", self.rbufsize)
		self.wfile = socket._fileobject(self.request, "wb", self.wbufsize)
		self.workingdir = '/tmp/p2v'

	def getSymlinkSize(self, path):
		''' The disk images are exposed using symbolic links.  Since os.path.getsize
		always returns 0 for symbolic links, I need to calculate the size in a
		different manner.  I assume that all the disk image files will end in 
		.img.
		'''
		f = open("%s" %(path), 'r')
		f.seek(0,2)
		size = f.tell()
		f.close()

		return str(size)

	def send_head(self):
		"""Common code for GET and HEAD commands.

		This sends the response code and MIME headers.

		Return value is either a file object (which has to be copied
		to the outputfile by the caller unless the command was HEAD,
		and must be closed by the caller under all circumstances), or
		None, in which case the caller has nothing further to do.

		"""
		path = self.translate_path(self.path)
		f = None
		if os.path.isdir(path):
			for index in "index.html", "index.htm":
				index = os.path.join(path, index)
				if os.path.exists(index):
					path = index
					break
				else:
					return self.list_directory(path)
		ctype = self.guess_type(path)
		try:
			# Always read in binary mode. Opening files in text mode may cause
			# newline translations, making the actual size of the content
			# transmitted *less* than the content-length!
			f = open(path, 'rb')
		except IOError:
			self.send_error(404, "File not found")
			return None

		# I since the disk image files are exposed as symbolic links, I need to use
		# A different method to calculate the size
		if self.path.endswith(".img"):
			size = self.getSymlinkSize(path)
		else:
			size = str(os.fstat(f.fileno())[6])

		self.send_response(200)
		self.send_header("Content-type", ctype)
		self.send_header("Content-Length", size)
		self.end_headers()
	        return f

	def do_GET(self):
		if self.path.endswith("reboot"):
			datafile = open("%s/%s" %(self.workingdir, self.path), 'r')
			self.send_response (200)
			self.send_header ("Content-type", "text/plain")
			self.end_headers ()
			while 1:
				c = datafile.read(1024)
				if c:
					self.wfile.write(c)
				else:
					datafile.close()
					break;
			os.popen4('/sbin/reboot')
			sys.exit(0)
		elif self.path.endswith(".img"):
			path = self.translate_path(self.path)
			size = self.getSymlinkSize(self.translate_path(self.path))
			datafile = open(path, 'r')
			self.send_response (200)
			self.send_header ("Content-type", "application/octet-stream")
			self.send_header ("Content-length", size)
			self.end_headers ()
			while 1:
				c = datafile.read(32768)
				if c:
					self.wfile.write(c)
				else:
					datafile.close()
					break;
		else:
			SimpleHTTPRequestHandler.do_GET(self)

	extensions_map = mimetypes.types_map.copy()
	extensions_map.update({
		'' : 'text/plain',
		'.img' : 'application/octet-stream',
		'.cfg' : 'text/plain',
	})

'''Utility function that will generate the ssl keys using the openssl command.
Defaults to creating the files in /tmp using the hostname returned by socket.gethostname().
Defaults can be overriden by providing the appropriate value:

p2vkey          the private key filename
p2vrequest      the request filename
p2vcert         the certificate filename
hostname        the hostname

Note that the contents of the files will be overwritten.

Returns the tuple (p2vkey, p2vrequest, p2vcert).
'''
def generateSSLkeys():
	# Some defaults
	p2vkey = '/tmp/p2vkey'
	p2vrequest = '/tmp/p2vrequest'
	p2vcert = '/tmp/p2vcertificate'
	hostname = socket.gethostname()

	# Find the openssl binary based on whether we are in a normal or install time environment
	opensslbin = ''
	if os.access('/usr/bin/openssl', os.X_OK):
		opensslbin = '/usr/bin/openssl'
	else:
		opensslbin = '/mnt/runtime/usr/bin/openssl'

	# Find the openssl.cnf file based on whether we are in a normal or install time environment
	if os.access('/etc/pki/tls/openssl.cnf', os.R_OK):
		cnffile = '/etc/pki/tls/openssl.cnf'
	else:
		cnffile = '/mnt/runtime/etc/pki/tls/openssl.cnf'

        # Generate the private key file
        (dummy, std_out_and_err) = os.popen4('%s genrsa -out %s 1024' %(opensslbin, p2vkey))

        # It is possible that the genrsa command has completed and returned but the key file has
        # not been generated, so I have to loop for a while until it is there or the request
        # command will fail
        while 1:
            if os.access(p2vkey, os.F_OK):
                if os.path.getsize(p2vkey) > 0:
                    break
                else:
                    time.sleep(1)
            else:
                time.sleep(1)

        # Generate the request.  All questions are left blank save the hostname.
        child = pexpect.spawn('%s req -new -key %s -out %s -config %s' %(opensslbin, p2vkey, p2vrequest, cnffile))
        child.setecho(False)
        child.expect('Country Name')
        child.sendline('.')
        child.expect('State or Province Name')
        child.sendline('.')
        child.expect('Locality Name')
        child.sendline('.')
        child.expect('Organization Name')
        child.sendline('.')
        child.expect('Organizational Unit Name')
        child.sendline('.')
        child.expect('Common Name')
        child.sendline(hostname)
        child.expect('Email Address')
        child.sendline('.')
        child.expect('A challenge password')
        child.sendline('.')
        child.expect('An optional company name')
        child.sendline('.')
        child.read()

        # Generate the certificate
        (dummy, std_out_and_err) = os.popen4('%s x509 -req -days 1000 -in %s -signkey %s -out %s' %(opensslbin, p2vrequest, p2vkey, p2vcert))

        # Return the key, request, and certificate files as a tuple
        return (p2vkey, p2vrequest, p2vcert)
