# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above. 
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

import os, logging
from snack import *
from p2vdata.P2VData import P2VData
from p2vkickstart.P2VHandlers import P2VHandlers
from p2vkickstart.P2VParser import P2VParser
from p2vtui.NetworkConfig import NetworkConfig
from p2vtui.SetupDisks import SetupDisks
from p2vtui.SetupOptions import SetupOptions
from p2vtui.PopulateWorkdir import PopulateWorkdir
from p2vtui.TransferMethod import TransferMethod
from p2vtui.SCPTransfer import SCPTransfer
from p2vtui.Finish import Finish
from p2vtui.Cancel import Cancel

def runP2V(anaconda, ksfile, logger = None, test = False):

	if not logger:
		logging.basicConfig(level=logging.DEBUG,
		format='%(asctime)s %(levelname)-8s %(message)s',
		datefmt='%a, %d %b %Y %H:%M:%S',
		filename='/tmp/p2v.log',
		filemode='a')
		logger = logging.getLogger("p2v")
 
	prompt = True
	currentstep = "NETWORK_CONFIG"
	p2vdata = P2VData()
	screen = None

	# if a kickstart file was passed to this function start by parsing that
	if ksfile:
		handlers = P2VHandlers(p2vdata)
		parser = P2VParser(p2vdata, handlers)
		parser.readKickstart(ksfile)

		# Set a flag so that user is only prompted if there is missing info
		prompt = False

	# Set these manually until bugzilla 6966 is resolved
	p2vdata.scptransfer = False
	p2vdata.ftptransfer = False
	p2vdata.managertransfer = True
	
	# These are the steps in the utility.
	p2vsteps = { "NETWORK_CONFIG" : NetworkConfig(previous = None, next = "SETUP_DISKS", test = test),
		"SETUP_DISKS" :  SetupDisks(previous = "NETWORK_CONFIG", next = "SETUP_OPTIONS"),
		"SETUP_OPTIONS" : SetupOptions(previous = "SETUP_DISKS", next = "POPULATE_WORKDIR"),
		"POPULATE_WORKDIR" : PopulateWorkdir(previous = "SETUP_OPTIONS", next = "FINISH"),
		#"TRANSFER_METHOD" : TransferMethod(previous = "SETUP_OPTIONS", next = "SCP_TRANSFER"),
		"SCP_TRANSFER" : SCPTransfer(previous = "TRANSFER_METHOD", next = "FINISH"),
		"FINISH" : Finish(previous = "TRANFER_METHOD", next = None),
		"CANCEL" : Cancel(previous = None, next = None)}

	logger.info('Starting Physical to Virtual migration utility')

	screen = SnackScreen()

	while not currentstep == None:
		logger.info("Proceeding to step %s" %(currentstep))
		(nextstep, prompt) = p2vsteps[currentstep].doStep(screen, p2vdata, logger, prompt)

		# If the user choose to cancel, save currentstep in the Cancel object so
		# that we can return if choice was a mistake
		if nextstep == "CANCEL":
			p2vsteps["CANCEL"].previous = currentstep

		currentstep = nextstep

	screen.finish()

if __name__ == '__main__':
	runP2V(None, None, logger = None, test = True)
	#runP2V(None, "test.cfg", logger = None, test = True)
