# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from __future__ import division
import os, time
from snack import *
from p2vtui.Step import Step
from p2vsslhttpserver.SecureHTTPServer import *
from p2vtransmit.SCPTransmit import SCPTransmit

class FileCopyScreen:

	def __init__(self, ui, filecount):
		self.filecount = filecount
		self.filescopied = 0
		self.bytecount = 0
		self.filename = None
		self.bytes = 0
		self.ui = ui

                self.theGrid = Grid(1, 4)
                self.theForm = Form()

                self.status = TextboxReflowed(60, "Copying file %d of %d" %(self.filescopied + 1, self.filecount))
		self.filestatus = TextboxReflowed(60, "Show me something")
                self.progress = Scale(60, filecount)
		self.fileprogress = Scale(60, 100)
                self.theGrid.setField(self.status, 0, 0, (0, 0, 0, 1))
                self.theGrid.setField(self.progress, 0, 1, (0, 0, 0, 1))
                self.theGrid.setField(self.filestatus, 0, 2, (0, 0, 0, 1))
                self.theGrid.setField(self.fileprogress, 0, 3, (0, 0, 0, 1))
                self.theForm.add(self.status)
                self.theForm.add(self.progress)
                self.theForm.add(self.filestatus)
                self.theForm.add(self.fileprogress)

                self.ui.gridWrappedWindow(self.theGrid, "Transfering files...")

	def bytescopied(self, bytes):
		self.bytes += bytes
		self.filestatus.setText("Copied %d of %d bytes of file %s" %(self.bytes, self.bytecount, self.filename))
		self.fileprogress.set((int)(self.bytes / self.bytecount * 100))

	def filecopied(self):
		self.filescopied += 1
		self.progress.set(self.filescopied)
		self.fileprogress.set(0)

	def nextFile(self, newfile, bytecount):
		self.filename = newfile
		self.bytecount = bytecount
		self.bytes = 0
		self.filestatus.setText("Copied %d of %d bytes of file %s" %(self.bytes, self.bytecount, self.filename))
		self.status.setText("Copying file %d of %d" %(self.filescopied + 1, self.filecount))
		
	def refresh(self):
		self.theForm.draw()
		self.ui.refresh()
		
class Finish(Step):

	def __init__(self, previous = "TRANFER_METHOD", next = None):
		Step.__init__(self, previous, next)

	def doStep(self, ui, data, logger, prompt):
		if data.scptransfer:
			logger.info('Copying files via SCP...')
			fcs = FileCopyScreen(ui, len(data.diskimages) +2)
			scp = SCPTransmit(fcs)

			try:
				scp.upload(data, logger)
				time.sleep(1)
                		ui.popWindow()
			except Exception, e:
				logger.error(e)
				ButtonChoiceWindow(ui, "Error",
					"An error was encountered when copying files via SCP:\n%s" %(e),
					buttons = [("OK", "OK", "F12")])
                		ui.popWindow()
				return ("SCP_TRANSFER", True)
		elif data.managertransfer:
			ui.finish()
			self.startwebserver(data, logger, prompt)
			ui = SnackScreen()

        	logger.info('Finishing p2v utility')
        	if prompt:
			ButtonChoiceWindow(ui, "Finished",
				"The P2V utility has completed successfully.  Press enter to exit the utility and reboot the system",
				buttons = [("OK", "OK", "F12")])
        	return (self.next, prompt)

	def startwebserver(self, data, logger, prompt):
        	logger.info('Starting web server')
	        print 'Starting web server'

                generatedKeys = False
                while not generatedKeys:
                    try:
        	        (key, req, cert) = generateSSLkeys()
                        generatedKeys = True
                    except Exception, e:
        	        logger.info('Failed to generate SSL keys: %s' %(e))
                        print 'Failed to generate SSL keys: %s' %(e)
        	        logger.error('Retrying...')
                        print 'Retying...'

	        currentdir = os.getcwd()
        	os.chdir('/tmp/p2v')
	        server_address = (data.ipaddr, 443) # (address, port)
        	httpd = SecureHTTPServer(server_address, SecureHTTPRequestHandler, key, cert)
	        sa = httpd.socket.getsockname()
        	logger.info("HTTPS web server is running on %s port %s..." %(sa[0], sa[1]))
	        print "HTTPS web server is running on %s port %s..." %(sa[0], sa[1])
        	print "Interrupt with control-C"
	        try:
                	httpd.serve_forever()
        	except KeyboardInterrupt:
	                logger.info("Stopping HTTPS web server")
                	print "Stopping HTTPS web server"
        	os.chdir(currentdir)
	        return "FINISH"

