# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

import os, time, popen2
from p2vtui.Step import Step
from snack import *

class PopulateWorkdir(Step):

	def __init__(self, previous = "SETUP_OPTIONS", next = "TRANSFER_METHOD"):
		Step.__init__(self, previous, next)

	def __findCommand(self, command, paths, args):
		for path in paths:
			if os.access('%s/%s' %(path, command), os.X_OK):
				return '%s/%s %s' %(path, command, args)
		return '%s %s' %(command, path)

	def makeDir(self, ui, theForm, data):
		self.status.setText("Making temporary directory to hold files")
		self.progress.set(1)
		theForm.draw()
		ui.refresh()

        	try:
                	os.mkdir('/tmp/p2v')
        	except OSError:
	                pass

	def makeVMcfg(self, ui, theForm, data):
		self.status.setText("Writing out vm.cfg file")
		self.progress.set(2)
		theForm.draw()
		ui.refresh()

	        f = open("/tmp/p2v/vm.cfg", "w")

		# Things that are written for everybody
        	f.write("acpi = 1\n")
	        f.write("apic = 1\n")
		f.write("keymap = \'en-us\'\n")
	        f.write("on_crash = \'restart\'\n")
        	f.write("on_reboot = \'restart\'\n")
        	f.write("device_model = \'/usr/lib/xen/bin/qemu-dm\'\n")
       		f.write("kernel = \'/usr/lib/xen/boot/hvmloader\'\n")
       		f.write("builder = \'hvm\'\n")
        	f.write("pae = 1\n")
       		f.write("serial = \'pty\'\n")
       		f.write("vif = [\'bridge=xenbr0,type=ioemu\']\n")
        	f.write("vnc = 1\n")
       		f.write("vncconsole = 1\n")
        	f.write("vnclisten = \'0.0.0.0\'\n")
        	f.write("vncunused = 1\n")

		# Written for everyone using info from data
       		f.write("vncpasswd = \'%s\'\n" %(data.consolepasswd))
	        f.write("memory = \'%s\'\n" %(data.vm_mem))
        	f.write("name = \'%s\'\n" %(data.vm_name))
	        f.write("vcpus = %s\n" %(data.vcpus))
		
		# Write out a disk line for every disk
		validdrives = ['a', 'b', 'c', 'd', 'e', 'f', 'g']
	        f.write("disk = [")
		for i in range(len(data.diskimages)):
			if i < 4:
				f.write("\'file:System-%s.img,hd%s,w\',\n" %(data.diskimages[i].device, validdrives[i]))
			elif i < 11:
				f.write("\'file:System-%s.img,sd%s,w\',\n" %(data.diskimages[i].device, validdrives[i - 4]))
                        else:
				f.write("#\'file:System-%s.img,sd%s,w\',\n" %(data.diskimages[i].device, validdrives[0]))
	        f.write("]\n")

        	f.close()

	def makeCatalog(self, ui, theForm, data):
		self.status.setText("Writing out vm.cfg file")
		self.progress.set(2)
		theForm.draw()
		ui.refresh()

		lspcipaths = ['/sbin', '/usr/sbin']
		ifconfigpaths = ['/sbin', '/usr/sbin']
		lvmpaths = ['/sbin', '/usr/sbin']

		commands = []
		commands.append(self.__findCommand('lspci', lspcipaths, '-nv'))
		commands.append(self.__findCommand('ifconfig', ifconfigpaths, '-a'))
		commands.append(self.__findCommand('lvm', lvmpaths, 'lvscan'))

		topline = '\n########################################\n'
		bottomline = '########################################\n'

		f = open("/tmp/p2v/catalog", "w")

		for cmd in commands:
			f.write(topline)
			f.write('### %s ###\n' %(cmd))
			f.write(bottomline)
			dummy, out = os.popen4(cmd)
			for line in out.readlines():
				f.write(line)

		f.close()

	def makeLinks(self, ui, theForm, data):
		self.status.setText("Creating symbolic links")
		self.progress.set(4)
		theForm.draw()
		ui.refresh()

		lnpaths = ['/bin', '/usr/bin']
	        lnbin = self.__findCommand('ln', lnpaths, '-s')

		for disk in data.diskimages:
			os.chmod(disk.path, 0644)
			dummy, out = os.popen4("%s %s /tmp/p2v/System-%s.img" %(lnbin, disk.path, disk.device))
			for line in out.readlines():
				self.logger.info(line)
			

	def makeManifest(self, ui, theForm, data):
		self.status.setText("Writing out MANIFEST file")
		self.progress.set(5)
		theForm.draw()
		ui.refresh()

		(std_out_and_err, dummy) = popen2.popen4("ls /tmp/p2v")
		lines = std_out_and_err.readlines()

        	f = open('/tmp/p2v/MANIFEST', 'w')
		for line in lines:
			if not line == 'MANIFEST':
				f.write(line)
        	f.close()
		
	def makeCGIBIN(self, ui, theForm, data):
		self.status.setText("Creating CGI-BIN directory")
		self.progress.set(6)
		theForm.draw()
		ui.refresh()
		
		try:
			os.mkdir('/tmp/p2v/cgi-bin')
			f = open('/tmp/p2v/cgi-bin/reboot', 'w')
			f.write('The server will now reboot')
			f.close()
		except OSError:
			pass

		
	def doStep(self, ui, data, logger, prompt):
		self.logger = logger
        	if not ui:
	                self.logger.info("UI was none so we are in test mode, skipping population")
                	return ("TRANSFER_METHOD", prompt)

		theGrid = Grid(1, 2)
		self.status = TextboxReflowed(60, "Now doing ...")
		self.progress = Scale(60, 6)
		theGrid.setField(self.status, 0, 0, (0, 0, 0, 1))
		theGrid.setField(self.progress, 0, 1, (0, 0, 0, 1))
		ui.gridWrappedWindow(theGrid, "Populating work directory")
		theForm = Form()
		theForm.add(self.status)
		theForm.add(self.progress)

		self.makeDir(ui, theForm, data)
		self.makeVMcfg(ui, theForm, data)
		self.makeCatalog(ui, theForm, data)
		self.makeLinks(ui, theForm, data)
		self.makeManifest(ui, theForm, data)
		self.makeCGIBIN(ui, theForm, data)

		time.sleep(1)
		ui.popWindow()

	        return (self.next, prompt)

