# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from p2vtui.Step import Step
from p2vdata.P2VDiskImageData import P2VDiskImageData
from snack import *
import os, re

class SetupDisks(Step):

	def __init__(self, previous = "NETWORK_CONFIG", next = "SETUP_OPTIONS"):
		Step.__init__(self, previous, next)
		self.disks = []

	def getDisks(self):
		# Find the cat binary
		if os.access("/usr/bin/cat", os.X_OK):
			catbin = "/usr/bin/cat"
		else:
			catbin = "cat"

		# Get the old-fashioned ide from /proc/partitions
		# /Z matches the end of the string, we only want whole disks not partitions
		ideregex = re.compile("xvd[a-z]\Z|hd[a-d]\Z")
		(std_in, std_out_and_err) = os.popen4('%s /proc/partitions' %(catbin))
		lines = std_out_and_err.readlines()

		# First two lines are the header info and can be thrown out
		for line in lines[2:]:
			device = line.split()[3]
			if ideregex.match(device):
				di = P2VDiskImageData(device, "IDE", "/dev/%s" %(device))
				self.disks.append(di)

		sdregex = re.compile("sd[a-z]*\Z")
		for line in lines[2:]:
			device = line.split()[3]
			if sdregex.match(device):
				di = P2VDiskImageData(device, "SCSI", "/dev/%s" %(device))
				self.disks.append(di)
			
		ccissregex = re.compile("cciss\/c[0-9]*d[0-9]*\Z")
		for line in lines[2:]:
			device = line.split()[3]
			if ccissregex.match(device):
				di = P2VDiskImageData(os.path.basename(device), "SCSI", "/dev/%s" %(device))
				self.disks.append(di)
		
		# Get any LVM Logical Volumes
		# Start any LVM disks that may exist
		#(std_in, std_out_and_err) = os.popen4('/usr/sbin/lvm vgchange -ay')
		#(std_in, std_out_and_err) = os.popen4('/usr/sbin/lvm vgmknods')
		#noLVMdisks = re.compile("No volume groups found")
		#(std_in, std_out_and_err) = os.popen4('/usr/sbin/lvm lvs')
		#lines = std_out_and_err.readlines()

		# First line is header or error indicating there are no groups
		#for line in lines[1:]:
			# Check for no groups message anyways to be safe
			#if not noLVMdisks.search(line):
				#logvol = line.split()[0] 
				#volgrp = line.split()[1]
				#path = "/dev/%s/%s" %(volgrp, logvol)
				#device = logvol
				#di = P2VDiskImageData(device, "LVM", path)
				#self.disks.append(di)

	def validateData(self, data):
		if len(data.diskimages) > 0:
			return True
		else:
			return False

	def inputsOK(self, ui):
		selections = self.tree.getSelection()
		if not selections:
			ButtonChoiceWindow(ui, "Error, no disk selected",
				"Please select at least one disk to transfer as part of the image",
				buttons = [ ("OK", "OK", "F12") ])
			return False
		else:
			return True

	def storeEntries(self, data):
		data.diskimages = self.tree.getSelection()

	def createGrid(self, ui, data):
                toplevel = GridFormHelp (ui, "Select the disks to include in the image", "P2V", 1, 2)

		self.tree = CheckboxTree(height = 5, scroll = 1)

		for disk in self.disks:
			self.logger.info('added item to tree')
			self.logger.info(disk)
			self.logger.info(disk.path)
			self.logger.info(disk.device)

			selected = False
			if disk in data.diskimages:
				selected = True
			self.tree.addItem(disk.device, (snackArgs['append'],), item=disk, selected=selected)

                self.bb = ButtonBar(ui, (("OK", "OK", "F12"), ("BACK", "BACK"), ("CANCEL", "CANCEL")))

                toplevel.add(self.tree, 0, 0, (0, 0, 0, 1), growx = 1)
                toplevel.add(self.bb, 0, 1, growx = 1)
                return toplevel
		
	def doStep(self, ui, data, logger, prompt):
                self.logger = logger

		if not self.disks:
        		self.logger.info('Scanning system for disk drives...')
			self.getDisks()

		if not prompt:
			# we are doing a kickstart install, just validate the data and continute
			if self.validateData(data):
				return (self.next, prompt)
			else:
				prompt = True
		
                while prompt:
                	toplevel = self.createGrid(ui, data)
                        result = toplevel.run()
                        rc = self.bb.buttonPressed(result)
                       	self.storeEntries(data)

                        if rc == "OK" or rc == "F12":
				if self.inputsOK(ui):
                                	ui.popWindow()
                                	return (self.next, prompt)
				else:
					continue
                        elif rc =="BACK":
                                return (self.previous, prompt)
                        elif rc == "CANCEL":
                                self.logger.info("User choose to CANCEL")
                                ui.popWindow()
                                return ("CANCEL", prompt)

	        return (self.next, prompt)
