# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from snack import *
from p2vtui.Step import Step
from p2vtui.common import checkField, invalidDialog


class SetupOptions(Step):

	def __init__(self, previous = "SETUP_DISKS", next = "POPULATE_WORKDIR"):
		Step.__init__(self, previous, next)
		self.minmem = 32
		self.maxmem = 262144
		self.minvcpus = 1
		self.maxvcpus = 32
                self.namemsg = "You need to supply a name for the virtual machine"
		self.memmsg = "The VM memory field must contain an integer"
		self.toomuchmemmsg = "The maximum memory that can be allocated to a VM is %sM" %(self.maxmem)
		self.toolittlememmsg = "The minimum memory that can be allocated to a VM is %sM" %(self.minmem)
		self.vcpusmsg = "The Virtual CPUs field must contain an integer"
		self.toomanyvcpusmsg = "The maximum number of Virtual CPUs that can be allocated to a VM is %s" %(self.maxvcpus)
		self.toofewvcpusmsg = "The minimum number of Virtual CPUs that can be allocated to a VM is %s" %(self.minvcpus)
                self.passmsg1 = "The console password fields are not the same"
                self.passmsg2 = "The console password field cannot be blank"

	def validateData(self, data):
		if not data.vm_name:
			return False

		if not data.consolepasswd:
			return False

		try:
			if data.vm_mem:
				dummy = int(data.vm_mem)
				if dummy < self.minmem or dummy > self.maxmem:
					return False
			else:
				return False

			if data.vcpus:
				dummy = int(data.vcpus)
				if dummy < self.minvcpus or dummy > self.maxvcpus:
					return False
			else:
				return False
		except ValueError:
			return False

		return True

	def inputsOK(self, ui):
                if not self.passentry.value() == self.confentry.value():
                        invalidDialog(ui, "Password", self.passmsg1)
                        return False
                else:
			try:
				dummy = int(self.mementry.value())
				if dummy > self.maxmem:
					invalidDialog(ui, "VM Memory", self.toomuchmemmsg)
					return False
				elif dummy < self.minmem:
					invalidDialog(ui, "VM Memory", self.toolittlememmsg)
					return False
			except ValueError:
                        	invalidDialog(ui, "VM Memory", self.memmsg)
                        	return False

			try:
				dummy = int(self.vcpusentry.value())
				if dummy > self.maxvcpus:
					invalidDialog(ui, "Virtual CPUs", self.toomanyvcpusmsg)
					return False
				if dummy < self.minvcpus:
					invalidDialog(ui, "Virtual CPUs", self.toofewvcpusmsg)
					return False
			except ValueError:
                        	invalidDialog(ui, "Virtual CPUs", self.vcpusmsg)
                        	return False

                       	return (checkField(ui, self.nameentry.value(), "VM name", self.namemsg, None, optional = False) and
                               	checkField(ui, self.passentry.value(), "Password", self.passmsg2, None, optional = False))

	def storeEntries(self, data):
		data.vm_name = self.nameentry.value()
		data.vm_mem = self.mementry.value()
		data.consolepasswd = self.passentry.value()
		data.vcpus = self.vcpusentry.value()

        def createSubGrid(self, data):
                theGrid = Grid(2, 5)

                theGrid.setField(Textbox(25, 1, "VM name:"), 0, 0)
                self.nameentry = Entry(20, scroll = 1)
                if data.vm_name:
                        self.nameentry.set(data.vm_name)
                theGrid.setField(self.nameentry, 1, 0)

                theGrid.setField(Textbox(25, 1, "VM Memory (in MB):"), 0, 1)
                self.mementry = Entry(20, scroll = 1)
                if data.vm_mem:
                        self.mementry.set(data.vm_mem)
                theGrid.setField(self.mementry, 1, 1)

                theGrid.setField(Textbox(25, 1, "Virtual CPUs:"), 0, 2)
                self.vcpusentry = Entry(20, scroll = 1)
                if data.vcpus:
                        self.vcpusentry.set(data.vcpus)
                theGrid.setField(self.vcpusentry, 1, 2)

                theGrid.setField(Textbox(25, 1, "Console Password:"), 0, 3)
                self.passentry = Entry(20, password = 1, scroll = 1)
                if data.consolepasswd:
                        self.passentry.set(data.consolepasswd)
                theGrid.setField(self.passentry, 1, 3)

                theGrid.setField(Textbox(25, 1, "Confirm console password:"), 0, 4)
                self.confentry = Entry(20, password = 1, scroll = 1)
                if data.consolepasswd:
                        self.confentry.set(data.consolepasswd)
                theGrid.setField(self.confentry, 1, 4)

                return theGrid

        def createGrid(self, ui, data):
                toplevel = GridFormHelp (ui, "Other parameters for VM", "P2V", 1, 2)

                self.bb = ButtonBar(ui, (("OK", "OK", "F12"), ("BACK", "BACK"), ("CANCEL", "CANCEL")))

                toplevel.add(self.createSubGrid(data), 0, 0, (0, 0, 0, 1), growx = 1)
                toplevel.add(self.bb, 0, 1, growx = 1)
                return toplevel

	def doStep(self, ui, data, logger, prompt):
                self.logger = logger

		if not prompt:
			# we are doing a kickstart install, just validate the data and move on
			self.logger.info("Validating options")
			if self.validateData(data):
				return (self.next, prompt)
			else:
				prompt = True

                while prompt:
                	toplevel = self.createGrid(ui, data)
                        result = toplevel.run()
                        rc = self.bb.buttonPressed(result)
                        if rc == "OK" or rc == "F12":
                                if self.inputsOK(ui):
                                        self.logger.info("User inputs seem valid")
                                        self.storeEntries(data)
                                        ui.popWindow()
                                        return (self.next, prompt)
                                else:
                                        continue
                        elif rc =="BACK":
                                self.storeEntries(data)
                                return (self.previous, prompt)
                        elif rc == "CANCEL":
                                self.logger.info("User choose to CANCEL")
                                ui.popWindow()
                                return ("CANCEL", prompt)

		return (self.next, prompt)
