# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

# The step class is an abstract class meant to define the interface that every
# step of the P2V utility should have.  Specific steps should be subclasses of
# this class.
class Step:

	# previous	-- Previous step in the utility
	# next		-- Next step in the utility
	# beenRun	-- Flag to indicate whether the step has already been run or not
	def __init__(self, previous = None, next = None, beenRun = False):
		self.previous = previous
		self.next = next
		self.beenRun = beenRun

	# All of the logic of the step should happen when the 'doStep' method is called
	# Subclasses should override this method with the appropriate steps
	# parameters are:
	# ui		-- The UI interface to use, for P2V version 1 this means a snack screen
	#		If this is None, the Step subclass should assume that it is being tested
	#		and should not display any UI elements
	# data		-- an instance of the P2VData class
	# logger	-- an instance of the python standard logging class to log messages to
	def doStep(self, ui, data, logger, prompt):
		pass
