# Copyright (c) 2008 Oracle Corporation.  All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#     - Redistributions of source code must retain the above
#       copyright notice, this list of conditions and the following
#       disclaimer.
#     - Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials
#       provided with the distribution.
#     - Neither the name of Oracle Corporation nor the names of its
#       contributors may be used to endorse or promote products
#       derived from this software without specific prior written
#       permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGE.
#
#Alternatively, the contents of this file may be used under the terms
#of the GNU General Public License version 2 (the "GPL") distributed
#with this softwarere in the file COPYING.GPL, in which case the
#provisions of the GPL are applicable instead of the above.
#
#If you wish to allow the use of your version of this file only under
#the terms of the GPL and not to allow others to use your version of
#this file under the license above, indicate your decision by deleting
#the provisions above and replace them with the notice and other
#provisions required by the GPL.  If you do not delete the provisions
#above, a recipient may use your version of this file under the above
#license or the GPL.

from snack import *
from p2vtui.Step import Step

class TransferMethod(Step):

	def __init__(self, previous = "SETUP_OPTIONS", next = "SCP_TRANSFER"):
		Step.__init__(self, previous, next)

	def validateData(self, data):
		if data.managertransfer:
			return True
		elif data.scptransfer or data.ftptransfer:
			return True
		else:
			return False

        def createGrid(self, ui, data):
                toplevel = GridFormHelp (ui, "P2V image transfer method", "P2V", 1, 3)
                toplevel.add(TextboxReflowed(40, "Please choose a method to use for image transfer."), 0, 0, (0, 0, 0, 1))


                self.bb = ButtonBar(ui, (("OK", "OK", "F12"), ("BACK", "BACK"), ("CANCEL", "CANCEL")))

		httpselected = True
		scpselected = False
		if data.scptransfer:
			httpselected = False
			scpselected = True
		
                # List of buttons for the RadioBar
                self.httpButton = ("Import image in OracleVM Manager (HTTPS)", "HTTPS", httpselected)
                self.scpButton = ("Send image via secure copy (SCP)", "SCP", scpselected)
                self.rb = RadioBar(ui, [self.httpButton, self.scpButton])

                toplevel.add(self.rb, 0, 1, (0, 0, 0, 1), growx = 1)
                toplevel.add(self.bb, 0, 2, growx = 1)
                return toplevel

        def doStep(self, ui, data, logger, prompt):
                self.logger = logger

		if not prompt:
			# We are doing a kickstart conversion, just validate the info
			# and return next step if OK
			if self.validateData(data):
				if data.managertransfer:
					return ("FINISH", prompt)
				else:
					return (self.next, prompt)
			else:
				prompt = True
			
                while prompt:
                	toplevel = self.createGrid(ui, data)
                        result = toplevel.run()
                        rc = self.bb.buttonPressed(result)

			choice = self.rb.getSelection()
			if choice == "HTTPS":
				data.managertransfer = True
				data.scptransfer = False
				data.ftptransfer = False
			elif choice == "SCP":
				data.managertransfer = False
				data.scptransfer = True
				data.ftptransfer = False
				 
                        if rc == "OK" or rc == "F12":
				ui.popWindow()
				if data.managertransfer:
					return ("FINISH", prompt)
				elif data.scptransfer:
					return ("SCP_TRANSFER", prompt)
			elif rc == "BACK":
				ui.popWindow()
				return (self.previous, prompt)
                        elif rc == "CANCEL":
                                self.logger.info("User choose to CANCEL")
                                ui.popWindow()
                                return ("CANCEL", prompt)

		return (self.next, prompt)
